/*
 *  DrixellasGame.h
 *  DrixellasDilemma
 *
 *  Created by Vince on 8/25/04.
 *  Copyright 2004 __MyCompanyName__. All rights reserved.
 *
 */

#include <Carbon/Carbon.h>
#include <vector>
#include "DrixellasConstants.h"
#include "Point.h"

typedef std::vector<tPoint>				tPointVector; 
typedef tPointVector::iterator			tPointVectorIter; 
typedef tPointVector::const_iterator	tPointVectorCIter; 

class tDrixellasPiece
{
public:
	SInt32	fColor;		//!< The game piece's color. 
	SInt32	fSymbol;	//!< The game piece's symbol. 
	
	tDrixellasPiece() : fColor(-1), fSymbol(-1) { /* empty */ }
	tDrixellasPiece(SInt32 color, SInt32 symbol) : fColor(color), fSymbol(symbol) { /* empty */ }
	
	inline bool	operator==(const tDrixellasPiece& piece);
	inline void	operator=(const tDrixellasPiece& piece);
	inline bool	mIsValid();
	inline void	mReset();
};

class tDrixellasGame
{
public:

	tDrixellasGame(); 
	~tDrixellasGame(); 
	
	void				mNewGame();
	void				mEndGame(); 
	tDrixellasPiece*	mCurrentGamePiece(); 
	tDrixellasPiece*	mGamePieceAt(UInt32 x, UInt32 y); 
	SInt32				mPlaceCurrentGamePieceAt(UInt32 x, UInt32 y);
	UInt32				mPiecesLeft(); 
	UInt32				mCurrentScore(); 
	bool				mUndoLastMove(); 
	UInt32				mTimeLeftForCurrentGamePiece(); 
	void				mToggleGamePaused(); 
	
	UInt32	mGridWidth()						{ return GRID_WIDTH; }
	UInt32	mGridHeight()						{ return GRID_HEIGHT; }
	bool	mValidLocation(UInt32 x, UInt32 y)	{ return x < mGridWidth() && y < mGridHeight(); }
	bool	mGameOver()							{ return fGameOver; }
	bool	mGameInProgress()					{ return !fUndoList.empty(); }
	bool	mGamePaused()						{ return fGamePaused; }
	UInt32	mPointsFromLastMove()				{ return fPointsFromLastMove; }
	
	const tPointVector&	mAvailableValidMoves()			{ return fAvailableValidMoves; }
	const tPointVector&	mPreviousAvailableValidMoves()	{ return fPreviousAvailableValidMoves; }
	
	void	mSkipCurrentPiece(); 
	
protected:
	
	tDrixellasPiece	fGamePieces[MAX_PUZZLE_PIECES];	//!< The game pieces. 
	tDrixellasPiece	fBoard[BOARD_SIZE];				//!< The game board. 
	
	UInt32	fCurrentPiece;		//!< The current piece that the player has to place. 
	UInt32	fScore;				//!< The player's current score. 
	bool	fGameOver;			//!< Flag indicating if the current game is over. 
	UInt32	fCurrentTimeLimit;
	UInt32	fLastPiecePlayedAtTime; 
	UInt32	fTimeElapsedBeforePause; 
	bool	fGamePaused; 
	UInt32	fPointsFromLastMove;
	
	struct tMove
	{
		tPoint	fMove;		//!< The move the player made. 
		UInt32	fPoints;	//!< The points the player earned for the move. 
		
		tMove(tPoint move, UInt32 points)
		{
			fMove = move; 
			fPoints = points; 
		}
	};
	
	std::vector<tMove>	fUndoList;	//!< Used to keep track of the player's moves so that they can be undone. 
	
	tPointVector	fAvailableValidMoves; 
	tPointVector	fPreviousAvailableValidMoves; 
	
	void	mPlaceStartingPiece(SInt32 color, SInt32 symbol, int x, int y);
	SInt32	mIsValidMove(UInt32 x, UInt32 y, UInt32& scoreModifier); 
	void	mDetermineAvailableMoves(); 
};

extern tDrixellasGame theGame; 


inline bool tDrixellasPiece::operator==(const tDrixellasPiece& piece)
{
	return piece.fColor == fColor && piece.fSymbol == fSymbol; 
}

inline void tDrixellasPiece::operator=(const tDrixellasPiece& piece)
{
	fColor = piece.fColor; 
	fSymbol = piece.fSymbol; 
}

inline bool tDrixellasPiece::mIsValid()
{
	return fColor > -1 && fSymbol > -1; 
}

inline void tDrixellasPiece::mReset()
{
	fColor = -1; 
	fSymbol = -1; 
}
